CREATE CACHED TABLE PRIMARY_KEY (
       KEY_NAME VARCHAR(50) NOT NULL
     , KEY_VALUE INTEGER DEFAULT '0' NOT NULL
     , PRIMARY KEY (KEY_NAME)
);

CREATE CACHED TABLE REPOSITORY (
       Name VARCHAR(50) NOT NULL
     , DESCRIPTION VARCHAR(250)
     , PRIMARY KEY (Name)
);

CREATE CACHED TABLE MESSAGE (
       ID INT NOT NULL
     , HASH_CODE INTEGER NOT NULL
     , MESSAGE_VALUE VARCHAR(2500) NOT NULL
     , CONSTRAINT UQ_MESSAGE_VALUE UNIQUE (MESSAGE_VALUE)
     , PRIMARY KEY (ID)
);
CREATE INDEX IX_MESSAGE_HASH_CODE ON MESSAGE (HASH_CODE);

CREATE CACHED TABLE PACKAGE (
       REPOSITORY VARCHAR(50) NOT NULL
     , NAME_ID INTEGER NOT NULL
     , PRIMARY KEY (REPOSITORY, NAME_ID)
     , CONSTRAINT FK_PACKAGE_REPOSITORY FOREIGN KEY (REPOSITORY)
                  REFERENCES REPOSITORY (Name) ON DELETE CASCADE
     , CONSTRAINT FK_PACKAGE_NAME FOREIGN KEY (NAME_ID)
                  REFERENCES MESSAGE (ID)
);

CREATE CACHED TABLE COMPILATION_UNIT (
       ID INTEGER NOT NULL
     , REPOSITORY VARCHAR(50) NOT NULL
     , PACKAGE INTEGER NOT NULL
     , PATH VARCHAR(250) NOT NULL
     , NAME VARCHAR(50) NOT NULL
     , COMPILATION_UNIT_SIZE BIGINT NOT NULL
     , CHECKSUM BIGINT NOT NULL
     , STORE_LEVEL SMALLINT DEFAULT '0' NOT NULL
     , ROOT VARCHAR(250) NOT NULL
     , REVISION VARCHAR(50)
     , CU_SOURCE LONGVARBINARY
     , PRIMARY KEY (ID)
     , CONSTRAINT FK_COMPILATION_UNIT_PACKAGE FOREIGN KEY (REPOSITORY, PACKAGE)
                  REFERENCES PACKAGE (REPOSITORY, NAME_ID) ON DELETE CASCADE
);

CREATE CACHED TABLE TYPE (
       COMPILATION_UNIT_ID INTEGER NOT NULL
     , NAME INTEGER NOT NULL
     , DECLARING_TYPE INTEGER
     , SUPERCLASS VARCHAR(250)
     , ENCLOSING_SCOPE_SIGNATURE VARCHAR(250)
     , IMPLEMENTOR INTEGER NOT NULL
     , STORE_LEVEL SMALLINT DEFAULT '0' NOT NULL
     , SUPERCLASS_SIGNATURE VARCHAR(250)
     , PRIMARY KEY (COMPILATION_UNIT_ID, NAME)
     , CONSTRAINT FK_TYPE_COMPILATION_UNIT FOREIGN KEY (COMPILATION_UNIT_ID)
                  REFERENCES COMPILATION_UNIT (ID) ON DELETE CASCADE
     , CONSTRAINT FK_TYPE_IMPLEMENTOR FOREIGN KEY (IMPLEMENTOR)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT "FK_TYPE_SQLC$DECLARING_TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
     , CONSTRAINT "FK_TYPE_SQLC$NAME" FOREIGN KEY (NAME)
                  REFERENCES MESSAGE (ID)
);

CREATE CACHED TABLE OPERATION (
       ID INTEGER NOT NULL PRIMARY KEY
     , COMPILATION_UNIT_ID INTEGER NOT NULL
     , DECLARING_TYPE INTEGER NOT NULL
     , NAME VARCHAR(50) NOT NULL
     , RETURN_TYPE VARCHAR(250)
     , PARAMETERS VARCHAR(1000) NOT NULL
     , MODIFIERS VARCHAR(50)
     , SIGNATURE VARCHAR(250) NOT NULL
     , CONSTRAINT "FK_OPERATION_SQLC$TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
);
CREATE INDEX IX_OPERATION_DECLARATION ON OPERATION (COMPILATION_UNIT_ID, DECLARING_TYPE, NAME);

CREATE CACHED TABLE SCAN (
       REPOSITORY VARCHAR(50) NOT NULL
     , ID INTEGER NOT NULL
     , SCAN_DATE DATETIME
     , SCAN_SIZE INTEGER
     , DESCRIPTION VARCHAR(250)
     , BYTE_SIZE BIGINT
     , CHECKSUM BIGINT
     , DEPENDENCY_LOADED BIT DEFAULT '0'
     , PRIMARY KEY (REPOSITORY, ID)
     , CONSTRAINT FK_SCAN_REPOSITORY FOREIGN KEY (REPOSITORY)
                  REFERENCES REPOSITORY (Name) ON DELETE CASCADE
);

CREATE CACHED TABLE COMPILATION_UNIT_SCAN (
       REPOSITORY VARCHAR(50) NOT NULL
     , SCAN_ID INTEGER NOT NULL
     , COMPILATION_UNIT_ID INTEGER NOT NULL
     , PRIMARY KEY (REPOSITORY, SCAN_ID, COMPILATION_UNIT_ID)
     , CONSTRAINT FK_COMPILATION_UNIT_SCAN_SCAN FOREIGN KEY (REPOSITORY, SCAN_ID)
                  REFERENCES SCAN (REPOSITORY, ID) ON DELETE CASCADE
     , CONSTRAINT FK_COMPILATION_UNIT_SCAN_COMPILATION_UNIT FOREIGN KEY (COMPILATION_UNIT_ID)
                  REFERENCES COMPILATION_UNIT (ID) ON DELETE CASCADE
);

CREATE CACHED TABLE FIELD (
       COMPILATION_UNIT_ID INTEGER NOT NULL
     , DECLARING_TYPE INTEGER NOT NULL
     , NAME VARCHAR(50) NOT NULL
     , TYPE VARCHAR(250)
     , SIGNATURE VARCHAR(250) NOT NULL
     , PRIMARY KEY (COMPILATION_UNIT_ID, DECLARING_TYPE, NAME)
     , CONSTRAINT "FK_FIELD_SQLC$TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
);

CREATE CACHED TABLE TYPE_DEPENDENCY (
       SUPPLIER_NAME INTEGER NOT NULL
     , CLIENT_CU_ID INTEGER NOT NULL
     , CLIENT_NAME INTEGER NOT NULL
     , SUPPLIER_PACKAGE INTEGER NOT NULL
     , CLIENT_PACKAGE INTEGER NOT NULL
     , PRIMARY KEY (SUPPLIER_NAME, CLIENT_CU_ID, CLIENT_NAME)
     , CONSTRAINT FK_DEPENDENCY_CLIENT FOREIGN KEY (CLIENT_CU_ID, CLIENT_NAME)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
     , CONSTRAINT FK_TYPE_DEPENDENCY_SUPPLIER_NAME FOREIGN KEY (SUPPLIER_NAME)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT FK_TYPE_DEPENDENCY_SUPPLIER_PACKAGE FOREIGN KEY (SUPPLIER_PACKAGE)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT FK_TYPE_DEPENDENCY_CLIENT_PACKAGE FOREIGN KEY (CLIENT_NAME)
                  REFERENCES MESSAGE (ID)
);

CREATE CACHED TABLE CALL_TRACE (
       ID INTEGER NOT NULL PRIMARY KEY
     , CALLER INTEGER NOT NULL
     , CALLEE_NAME VARCHAR(250)
     , CALLEE INTEGER
     , CONSTRAINT "FK_CALLED_OPERATION_SQLC$CALLER" FOREIGN KEY (CALLER)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
     , CONSTRAINT "FK_CALL_TRACE_SQLC$CALLEE" FOREIGN KEY (CALLEE)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
);

CREATE CACHED TABLE OVERRIDE (
       ID INTEGER NOT NULL PRIMARY KEY
     , SPECIFICATION INTEGER NOT NULL
     , IMPLEMENTATION INTEGER NOT NULL
     , CONSTRAINT FK_OVERRIDE_IMPLEMENTATION FOREIGN KEY (IMPLEMENTATION)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
     , CONSTRAINT "FK_OVERRIDE_SQLC$SPECIFICATION" FOREIGN KEY (SPECIFICATION)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
);

CREATE CACHED TABLE INTERFACE (
       COMPILATION_UNIT_ID INTEGER NOT NULL
     , DECLARING_TYPE INTEGER NOT NULL
     , INTERFACE_NAME INTEGER NOT NULL
     , SIGNATURE VARCHAR(250)
     , PRIMARY KEY (COMPILATION_UNIT_ID, DECLARING_TYPE, INTERFACE_NAME)
     , CONSTRAINT FK_INTERFACE_NAME FOREIGN KEY (INTERFACE_NAME)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT "FK_INTERFACE_SQLC$TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
);

