CREATE TABLE REPOSITORY (
       Name VARCHAR2(50) NOT NULL
     , DESCRIPTION VARCHAR2(250)
     , PRIMARY KEY (Name)
);

CREATE TABLE MESSAGE (
       ID INT NOT NULL
     , HASH_CODE NUMBER(10) NOT NULL
     , MESSAGE_VALUE VARCHAR2(2500) NOT NULL
     , CONSTRAINT UQ_MESSAGE_VALUE UNIQUE (MESSAGE_VALUE)
     , PRIMARY KEY (ID)
);
CREATE INDEX IX_MESSAGE_HASH_CODE ON MESSAGE (HASH_CODE);

CREATE TABLE PACKAGE (
       REPOSITORY VARCHAR2(50) NOT NULL
     , NAME_ID NUMBER(10) NOT NULL
     , PRIMARY KEY (REPOSITORY, NAME_ID)
     , CONSTRAINT FK_PACKAGE_REPOSITORY FOREIGN KEY (REPOSITORY)
                  REFERENCES REPOSITORY (Name) ON DELETE CASCADE
     , CONSTRAINT FK_PACKAGE_NAME FOREIGN KEY (NAME_ID)
                  REFERENCES MESSAGE (ID)
);

CREATE TABLE COMPILATION_UNIT (
       ID NUMBER(10) NOT NULL PRIMARY KEY
     , REPOSITORY VARCHAR2(50) NOT NULL
     , PACKAGE NUMBER(10) NOT NULL
     , PATH VARCHAR2(1000) NOT NULL
     , NAME VARCHAR2(200) NOT NULL
     , COMPILATION_UNIT_SIZE NUMBER(20) NOT NULL
     , CHECKSUM NUMBER(20) NOT NULL
     , STORE_LEVEL SMALLINT DEFAULT '0' NOT NULL
     , ROOT VARCHAR2(1000) NOT NULL
     , REVISION VARCHAR2(50)
     , CU_SOURCE BLOB
     , CONSTRAINT FK_COMPILATION_UNIT_PACKAGE FOREIGN KEY (REPOSITORY, PACKAGE)
                  REFERENCES PACKAGE (REPOSITORY, NAME_ID) ON DELETE CASCADE
);

CREATE TABLE TYPE (
       COMPILATION_UNIT_ID NUMBER(10) NOT NULL
     , NAME NUMBER(10) NOT NULL
     , DECLARING_TYPE NUMBER(10)
     , SUPERCLASS VARCHAR2(2500)
     , ENCLOSING_SCOPE_SIGNATURE VARCHAR2(2500)
     , IMPLEMENTOR NUMBER(10) NOT NULL
     , SUPERCLASS_SIGNATURE VARCHAR2(2500)
     , STORE_LEVEL SMALLINT DEFAULT 0 NOT NULL
     , PRIMARY KEY (COMPILATION_UNIT_ID, NAME)
     , CONSTRAINT FK_TYPE_COMPILATION_UNIT FOREIGN KEY (COMPILATION_UNIT_ID)
                  REFERENCES COMPILATION_UNIT (ID) ON DELETE CASCADE
     , CONSTRAINT FK_TYPE_NAME FOREIGN KEY (NAME)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT FK_TYPE_IMPLEMENTOR FOREIGN KEY (IMPLEMENTOR)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT "FK_TYPE_SQLC$DECLARING_TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
);

CREATE TABLE OPERATION (
       ID NUMBER(10) NOT NULL IDENTITY PRIMARY KEY
     , COMPILATION_UNIT_ID NUMBER(10) NOT NULL
     , DECLARING_TYPE NUMBER(10) NOT NULL
     , NAME VARCHAR2(200) NOT NULL
     , RETURN_TYPE VARCHAR2(250)
     , PARAMETERS VARCHAR2(1000)
     , MODIFIERS VARCHAR2(100)
     , SIGNATURE VARCHAR2(2500) NOT NULL
     , CONSTRAINT "FK_OPERATION_SQLC$TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
);

CREATE TABLE SCAN (
       REPOSITORY VARCHAR2(50) NOT NULL
     , ID NUMBER(10) NOT NULL
     , SCAN_DATE DATETIME
     , SCAN_SIZE NUMBER(10)
     , DESCRIPTION VARCHAR2(250)
     , BYTE_SIZE NUMBER(20)
     , CHECKSUM NUMBER(20)
     , DEPENDENCY_LOADED BIT DEFAULT '0'
     , PRIMARY KEY (REPOSITORY, ID)
     , CONSTRAINT FK_SCAN_REPOSITORY FOREIGN KEY (REPOSITORY)
                  REFERENCES REPOSITORY (Name) ON DELETE CASCADE
);

CREATE TABLE COMPILATION_UNIT_SCAN (
       REPOSITORY VARCHAR2(50) NOT NULL
     , SCAN_ID NUMBER(10) NOT NULL
     , COMPILATION_UNIT_ID NUMBER(10) NOT NULL
     , PRIMARY KEY (REPOSITORY, SCAN_ID, COMPILATION_UNIT_ID)
     , CONSTRAINT FK_COMPILATION_UNIT_SCAN_SCAN FOREIGN KEY (REPOSITORY, SCAN_ID)
                  REFERENCES SCAN (REPOSITORY, ID) ON DELETE CASCADE
     , CONSTRAINT FK_COMPILATION_UNIT_SCAN_COMPILATION_UNIT FOREIGN KEY (COMPILATION_UNIT_ID)
                  REFERENCES COMPILATION_UNIT (ID) ON DELETE CASCADE
);

CREATE TABLE FIELD (
       COMPILATION_UNIT_ID NUMBER(10) NOT NULL
     , DECLARING_TYPE NUMBER(10) NOT NULL
     , NAME VARCHAR2(50) NOT NULL
     , TYPE VARCHAR2(250)
     , SIGNATURE VARCHAR2(250) NOT NULL
     , PRIMARY KEY (COMPILATION_UNIT_ID, DECLARING_TYPE, NAME)
     , CONSTRAINT "FK_FIELD_SQLC$TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
);

CREATE TABLE TYPE_DEPENDENCY (
       SUPPLIER_NAME NUMBER(10) NOT NULL
     , CLIENT_CU_ID NUMBER(10) NOT NULL
     , CLIENT_NAME NUMBER(10) NOT NULL
     , SUPPLIER_PACKAGE NUMBER(10) NOT NULL
     , CLIENT_PACKAGE NUMBER(10) NOT NULL
     , PRIMARY KEY (SUPPLIER_NAME, CLIENT_CU_ID, CLIENT_NAME)
     , CONSTRAINT FK_DEPENDENCY_CLIENT FOREIGN KEY (CLIENT_CU_ID, CLIENT_NAME)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
     , CONSTRAINT FK_TYPE_DEPENDENCY_SUPPLIER_NAME FOREIGN KEY (SUPPLIER_NAME)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT FK_TYPE_DEPENDENCY_SUPPLIER_PACKAGE FOREIGN KEY (SUPPLIER_PACKAGE)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT FK_TYPE_DEPENDENCY_CLIENT_PACKAGE FOREIGN KEY (CLIENT_NAME)
                  REFERENCES MESSAGE (ID)
);

CREATE TABLE CALL_TRACE (
       ID NUMBER(10) NOT NULL IDENTITY PRIMARY KEY
     , CALLER NUMBER(10) NOT NULL
     , CALLEE_NAME VARCHAR2(250)
     , CALLEE NUMBER(10)
     , CONSTRAINT FK_CALLED_OPERATION_CALLER FOREIGN KEY (CALLER)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
     , CONSTRAINT FK_CALL_TRACE_CALLEE FOREIGN KEY (CALLEE)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
);

CREATE TABLE OVERRIDE (
       ID NUMBER(10) NOT NULL IDENTITY PRIMARY KEY
     , SPECIFICATION NUMBER(10)
     , SPECIFICATION_SIGNATURE VARCHAR2(250)
     , IMPLEMENTATION NUMBER(10) NOT NULL
     , CONSTRAINT FK_OVERRIDE_SPECIFICATION FOREIGN KEY (SPECIFICATION)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
     , CONSTRAINT FK_OVERRIDE_IMPLEMENTATION FOREIGN KEY (IMPLEMENTATION)
                  REFERENCES OPERATION (ID) ON DELETE CASCADE
);

CREATE TABLE INTERFACE (
       COMPILATION_UNIT_ID NUMBER(10) NOT NULL
     , DECLARING_TYPE NUMBER(10) NOT NULL
     , INTERFACE_NAME NUMBER(10) NOT NULL
     , SIGNATURE VARCHAR2(250)
     , PRIMARY KEY (COMPILATION_UNIT_ID, DECLARING_TYPE, INTERFACE_NAME)
     , CONSTRAINT FK_INTERFACE_NAME FOREIGN KEY (INTERFACE_NAME)
                  REFERENCES MESSAGE (ID)
     , CONSTRAINT "FK_INTERFACE_SQLC$TYPE" FOREIGN KEY (COMPILATION_UNIT_ID, DECLARING_TYPE)
                  REFERENCES TYPE (COMPILATION_UNIT_ID, NAME) ON DELETE CASCADE
);

